
import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;

abstract public class AntennaeGUI extends PhysicsApplet{

    /*  wave fields  */
  double[][] phi0;
  double[][] phi;
  int[][] State;
    /* antenna coordinates */
  int nantennae;
  int[] iphase;
  int [] ixposition;
  int [] iyposition;
  PointDisplay Dnear;
  ArrayDisplay Dfar;
  TitleBanner Legend;
     // mouse modes:
  static final int phase0Mode = 0; 
  static final int phase90Mode = 1;
  static final int phase180Mode = 2;
  static final int phase270Mode = 3;
  static final int eraseMode = 4;
  static final int measureMode = 5;
     // button codes:
  static final int ResetCode = 1;
  static final int StartCode = 3;
  static final int ScrollBarCode = 4;
  static final int tooManyCode = 5;
  PhysicsScrollbar PSB;
  BoldLabel SBLabel;
  double scale;

  AntennaeGUI(){
    super("Antennae Applet",100,100,2,60);
    phi0 = new double[Nx+1][Ny+1];
    phi = new double[Nx+1][Ny+1];
    State = new int[Nx+1][Ny+1];
    nantennae = 0;
    iphase = new int[100];
    ixposition = new int[100];
    iyposition = new int[100];
    scale = 1.0;
    for (int i = 0; i <= Nx; i++){
      for (int j = 0; j <= Ny; j++){
         phi0[i][j] = 0.0;
         phi[i][j] = 0.0;
       }
    }
  }

  double xposition(int m){
      if (m < 1 || m >nantennae){
	Legend.write(
		" You asked for info about a missing antenna");
        return 0.0;
      } 
      return 0.05 * (ixposition[m] - 50);
  }
      
  double yposition(int m){
      if (m < 1 || m >nantennae){
	Legend.write(
		" You asked for info about a missing antenna");
        return 0.0;
      } 
      return -0.05 * (iyposition[m] - 50);
  }
      
  double phase(int m){
      if (m < 1 || m >nantennae){
	Legend.write(
		" You asked for info about a missing antenna");
        return 0.0;
      } 
      return 0.5 * Math.PI * iphase[m];
  }

  void resetArrays(){
    for (int mx = 0; mx <= Nx; mx++){
       for (int my = 0; my <= Ny; my++){
         phi[mx][my] = 0.0;
       }
     }
  }

  void resetAll(){
    for (int mx = 0; mx <= Nx; mx++){
       for (int my = 0; my <= Ny; my++){
         phi[mx][my] = 0.0;
       }
    }
    nantennae = 0;
    refreshPicture();
  }

  class PointDisplay extends ArrayDisplay{

     public PointDisplay(double[][]  AA, int[][] id,
           Color MaxColor, Color BaseColor){
	 super(AA,id,MaxColor,BaseColor);
     }
     
     public void mouseClicked(MouseEvent e){
      if (mouseMode >= phase0Mode && mouseMode <= phase270Mode){
	  if (nantennae >= 99){ 
            doAction(tooManyCode);
          } else {
            int i = (e.getX()-leftmargin)/pixelsize -1;
            int j = e.getY()/pixelsize -1;
            if ( i > 0 && i < Nx-1 && j > 0 && j < Ny-1){
	     nantennae++;
             ixposition[nantennae] = i;
             iyposition[nantennae] = j;
             iphase[nantennae] = mouseMode;
            }
          }
      }
      if (mouseMode == eraseMode){
         int i = (e.getX()-leftmargin)/pixelsize -1;
         int j = e.getY()/pixelsize -1;
	 int k = 0;
         while (k < nantennae){
	    k++;
            if ( i-5 < ixposition[k] && i+5 > ixposition[k] &&
		 j-5 < iyposition[k] && j+5 > iyposition[k]){
		for (int l = k+1; l <= nantennae; l++){
		  ixposition[l-1] = ixposition[l];
                  iyposition[l-1] = iyposition[l];
                  iphase[l-1] = iphase[l];
                }
                nantennae--;
             }
	 }
      }
      refreshPicture();
     }

     public void mouseDragged(MouseEvent e){}

     public void paint(Graphics g){
       g.setClip(leftmargin,0,(Nx+3)*pixelsize, (Ny+3)*pixelsize);
       g.setColor(Color.black);
       g.fillRect(leftmargin,0,2*pixelsize,(Ny+3)*pixelsize); 
       g.fillRect(leftmargin,0,(Nx+3)*pixelsize, 2*pixelsize); 
       g.fillRect(leftmargin,(Ny+1)*pixelsize,(Nx+3)*pixelsize,
                                              2*pixelsize); 
       g.fillRect(leftmargin+(Nx+1)*pixelsize, 0,2*pixelsize,
                                              (Ny+3)*pixelsize);
       g.setColor(Color.white);
       g.fillRect(leftmargin+2*pixelsize,2*pixelsize,
                                 (Nx-1)*pixelsize,(Ny-1)*pixelsize);
       for (int k = 1; k <= nantennae; k++){
	   if (iphase[k] == phase0Mode)  g.setColor(Color.orange);
	   if (iphase[k] == phase90Mode)  g.setColor(Color.green);
	   if (iphase[k] == phase180Mode)  g.setColor(Color.blue);
	   if (iphase[k] == phase270Mode)  g.setColor(Color.magenta);
           int truex = leftmargin + (ixposition[k]+1)*pixelsize;
           int truey = (iyposition[k]+1)*pixelsize;
           g.fillOval(truex-5,truey-5, 10,10);
       }
       g.setColor(Color.black);
       for(int i = 1 ; i <= Nx/10 -1; i++){
          for (int j = 1; j <= Ny/10 -1; j++){
              g.fillRect(leftmargin + (10*i + 1)* pixelsize + halfpix, 
                  (10*j+1)*pixelsize + halfpix, 2, 2);
           }
      }
    }


  }

  void buildPicture(){
    Dnear = new PointDisplay(phi0,State, Color.red,Color.cyan);
    Dfar = new ArrayDisplay(phi,State, Color.red,Color.cyan);
    Picture.add(Dfar,"East");
    Picture.add(Dnear,"West");
    Legend = new TitleBanner(" ", 16);
    Picture.add(Legend,"South");
  }

  void refreshPicture(){
    Dnear.refresh();
    Dfar.refresh();
  }
  
  void buildControls(){
    Controls.setLayout(new GridLayout(0,4,10,10));
    ModeButton B1 = new ModeButton("0 degrees", phase0Mode);
    Controls.add(B1);
    ModeButton B2 = new ModeButton("90 degrees", phase90Mode);
    Controls.add(B2);   
    ModeButton B3 = new ModeButton("180 degrees", phase180Mode);
    Controls.add(B3);
    ModeButton B4 = new ModeButton("270 degrees", phase270Mode);
    Controls.add(B4);   
    ModeButton B5 = new ModeButton("Erase", eraseMode);
    Controls.add(B5);   
    CommandButton B6 = new CommandButton("Compute", StartCode);
    Controls.add(B6);
    ModeButton B7 = new ModeButton("Measure Field", measureMode);
    Controls.add(B7);
    CommandButton B8 = new CommandButton("Reset Sources", ResetCode);
    Controls.add(B8);
    BoldLabel B9 = new BoldLabel("  Intensity Scale");
    Controls.add(B9);
    PSB = new PhysicsScrollbar(scale, 0.2, 20.0, ScrollBarCode);
    Controls.add(PSB);
    SBLabel = new BoldLabel("   "+scale);
    Controls.add(SBLabel);
  }

  Color findColor(double A, int S){
    int colorm =  (int) ( A/ scale);
    if (colorm > 10) colorm = 10;
    return Dfar.DisplayColors[colorm];
  }

  void writeFieldValue(double Val, int mode){
    if (mode == measureMode) Legend.write(" Intensity = " + Val);
  }

  void doAction(int Code){
    switch(Code){
      case ResetCode:
        resetAll();
        break;;
     case StartCode:
        startThread();
        break;
     case ScrollBarCode:
        scale = PSB.getReading();
        SBLabel.setText("   "+scale);
        break;
     case tooManyCode:
        Legend.write(" The maximum number of antennae is 99.");
        break;
     default:
        break;
    } 
  }

  void writeVectorValue(double Vx, double Vy, int mode){}
  void setArrays(int i, int j, int mode){}
  void plot(){}
}
      
    










