
import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;

abstract public class DielectricGUI extends PhysicsApplet{

  double[][] phi;
  double epsilon;
  int[][] State;
  ArrayDisplay D;
  TitleBanner Legend;
  double CathodeV = 100.0;
     // mouse modes:
  static final int NormalMode = 0;
  static final int CathodeMode = 1;
  static final int GroundMode = 2;
  static final int EraseMode = 3;
  static final int DielectricMode = 4;
  static final int EraseDielectricMode = 5;
  static final int MeasureMode = 6;
     // button codes:
  static final int FieldResetCode = 1;
  static final int AllResetCode = 2;
  static final int MeasureECode = 3;
  static final int StartCode = 4;
  static final int StopCode = 5;
  static final int ScrollBarCode = 6;
  PhysicsScrollbar PSB;
  BoldLabel SBLabel;
    // array states:
  static final int NormalState = 0;
  static final int DielectricState = -1;
  static final int CathodeState = 1;
  static final int GroundState = 2;
  

  DielectricGUI(){
    super("Dielectric Applet",100,100,4,100);
    epsilon = 5.0;
    phi = new double[Nx+1][Ny+1];
    State = new int[Nx+1][Ny+1];
    for (int i = 0; i <= Nx; i++){
      for (int j = 0; j <= Ny; j++){
         phi[i][j] = 0.0;
         State[i][j] = 0;
       }
    }
  }

  void resetArrays(){
      for (int mx = 0; mx <= Nx; mx++){
        for (int my = 0; my <= Ny; my++){
          if (State[mx][my] <= NormalState)  phi[mx][my] = 0.0;
        }
      }
      refreshPicture();
    }

    void resetAll(){
      for (int mx = 0; mx <= Nx; mx++){
        for (int my = 0; my <= Ny; my++){
         phi[mx][my] = 0.0;
         State[mx][my] = 0;
        }
      }
      refreshPicture();
    }

  void buildPicture(){
    D = new ArrayDisplay(phi,State, Color.red,Color.cyan);
    Picture.add(D,"Center");
    Legend = new TitleBanner(" ", 16);
    Picture.add(Legend,"South");
  }

  void refreshPicture(){
    D.refresh();
  }
  
  void buildControls(){
    Controls.setLayout(new GridLayout(0,4,10,10));
    ModeButton B1 = new ModeButton("Cathode", CathodeMode);
    Controls.add(B1);
    ModeButton B2 = new ModeButton("Ground", GroundMode);
    Controls.add(B2);   
    ModeButton B3 = new ModeButton("Erase", EraseMode);
    Controls.add(B3);   
    CommandButton B4 = new CommandButton("Reset All", AllResetCode);
    Controls.add(B4);
    ModeButton B5 = new ModeButton("Dielectric", DielectricMode);
    Controls.add(B5);
    ModeButton B6 = new ModeButton("Erase Dielectric", EraseDielectricMode);
    Controls.add(B6);   
    CommandButton B7 = new CommandButton("Reset Fields", FieldResetCode);
    Controls.add(B7);
    Label B8 = new Label(" ");
    Controls.add(B8);
    CommandButton B9 = new CommandButton("Solve Laplace", StartCode);
    Controls.add(B9);
    CommandButton B10 = new CommandButton("Stop",StopCode);
    Controls.add(B10);
    ModeButton B11 = new ModeButton("Measure Voltage", MeasureMode);
    Controls.add(B11);
    CommandButton B12 = new CommandButton("Compute Energy",MeasureECode);
    Controls.add(B12);
    BoldLabel B13 = new BoldLabel("  Dielectric Constant");
    Controls.add(B13);
    PSB = new PhysicsScrollbar(epsilon, 1.0, 21.0, ScrollBarCode);
    Controls.add(PSB);
    SBLabel = new BoldLabel("   "+epsilon);
    Controls.add(SBLabel);
  }

  Color findColor(double A, int S){
    if (S == CathodeState)  return Color.magenta;
    if (S == GroundState)   return Color.black;
    int colorm =  (int) ( 10.0 * A/CathodeV);
    if (S == DielectricState) return D.altDisplayColors[colorm];
    return D.DisplayColors[colorm];
  }

  void writeFieldValue(double Val, int mode){
    if (mode == MeasureMode) Legend.write(" Voltage = " + Val);
  }

  boolean normal(int i, int j){
    if (State[i][j] == NormalState || State[i][j] == DielectricState)
	    return true;
    return false;
  }

  double getepsilon(int i, int j){
    if (State[i][j] == DielectricState) return epsilon;
    return 1.0;
  }

  void setArrays(int i, int j, int mode){
    switch(mode){
      case CathodeMode:
        phi[i][j] = CathodeV;
        State[i][j] = CathodeState;
        break;
      case GroundMode:
        phi[i][j] = 0.0;
        State[i][j] = GroundState;
        break;
      case EraseMode:
        phi[i][j] = 0.0;
        State[i][j] = NormalState;
        break;
      case DielectricMode:
        if (State[i][j] == NormalState) State[i][j] = DielectricState;
        break;
      case EraseDielectricMode:
        if (State[i][j] == DielectricState) State[i][j] = NormalState;
        break;
      default:
	break;
    }
  }

  void doAction(int Code){
    switch(Code){
      case FieldResetCode:
        resetArrays();
        break;;
     case AllResetCode:
        resetAll();
        break;
     case MeasureECode:
        double E = Energy();
        Legend.write("Energy = "+E);
        break;
     case StartCode:
        startThread();
        break;
     case StopCode:
        stopThread();
        break;
     case ScrollBarCode:
        double neweval = PSB.getReading();
        epsilon = neweval;
        SBLabel.setText(epsilon+" ");
        break;
     default:
        break;
    } 
  }

  abstract double Energy();

  void writeVectorValue(double Vx, double Vy, int mode){}
    void plot(){}
}
      
    










