
import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;

abstract public class DiffractionGUI extends PhysicsApplet{

    /*  wave fields  */
  double[][] phi0;
  double[][] phi;
  int[][] State;
  int[][] State2;
    /* antenna coordinates */
  int nantennae;
  ArrayDisplay Dnear;
  ArrayDisplay Dfar;
  TitleBanner Legend;
     // mouse modes:
  static final int drawMode = 3;
  static final int eraseMode = 4;
  static final int measureMode = 5;
     // button codes:
  static final int ResetCode = 1;
  static final int StartCode = 3;
  static final int ScrollBarCode = 4;
    // state codes:
  static final int blackState = 0;
  static final int whiteState = 1;
  static final int farState = 2;
  PhysicsScrollbar PSB;
  BoldLabel SBLabel;
  double scale;
  Color [] myDisplayColors;

  DiffractionGUI(){
    super("Diffraction Applet",100,100,3,60);
    phi0 = new double[Nx+1][Ny+1];
    phi = new double[Nx+1][Ny+1];
    State = new int[Nx+1][Ny+1];    
    State2 = new int[Nx+1][Ny+1];
    scale = 1.0;
    for (int i = 0; i <= Nx; i++){
      for (int j = 0; j <= Ny; j++){
         phi0[i][j] = 0.0;
         phi[i][j] = 0.0;
         State[i][j] = 0;
         State2[i][j] = farState;
       }
    }
    myDisplayColors = new Color[11];
    for (int k = 0; k < 11; k++){
	myDisplayColors[k] = new Color(k*25, k*k*2 , k*k*2);
    }
  }

  void resetArrays(){
    for (int mx = 0; mx <= Nx; mx++){
       for (int my = 0; my <= Ny; my++){
         phi[mx][my] = 0.0;
       }
     }
  }

  void resetAll(){
    for (int mx = 0; mx <= Nx; mx++){
       for (int my = 0; my <= Ny; my++){
         phi[mx][my] = 0.0;
         State[mx][my] = 0;
       }
    }
    refreshPicture();
  }


  void buildPicture(){
    Dnear = new ArrayDisplay(phi0,State, Color.red,Color.cyan);
    Dfar = new ArrayDisplay(phi,State2, Color.red,Color.cyan);
    Picture.add(Dfar,"East");
    Picture.add(Dnear,"West");
    Legend = new TitleBanner(" ", 16);
    Picture.add(Legend,"South");
  }

  void refreshPicture(){
    Dnear.refresh();
    Dfar.refresh();
  }
  
  void buildControls(){
    Controls.setLayout(new GridLayout(0,4,10,10));
    ModeButton B1 = new ModeButton("Draw Aperture", drawMode);
    Controls.add(B1);
    ModeButton B2 = new ModeButton("Erase Aperture", eraseMode);
    Controls.add(B2);   
    ModeButton B3 = new ModeButton("Measure Field", measureMode);
    Controls.add(B3);
    CommandButton B4 = new CommandButton("Reset", ResetCode);
    Controls.add(B4);
    CommandButton B5 = new CommandButton("Compute", StartCode);
    Controls.add(B5);
    BoldLabel B6 = new BoldLabel("  Intensity Scale");
    Controls.add(B6);
    PSB = new PhysicsScrollbar(scale, 0.2, 20.0, ScrollBarCode);
    Controls.add(PSB);
    SBLabel = new BoldLabel("   "+scale);
    Controls.add(SBLabel);
  }

  Color findColor(double A, int S){
      if (S == 0) return Color.black;
      if (S == 1) return Color.white;
      int colorm =  (int) Math.log( A/ scale);
      if (colorm < 0) colorm = 0;
      if (colorm > 10) colorm = 10;
      return myDisplayColors[colorm];
  }

  void writeFieldValue(double Val, int mode){
    if (mode == measureMode) Legend.write(" Intensity = " + Val);
  }

  void setArrays(int i, int j, int mode){
    if (mode == drawMode){
       State[i][j] = whiteState;
    } else if (mode == eraseMode){
	State[i][j] = blackState;
    }
  }


  void doAction(int Code){
    switch(Code){
      case ResetCode:
        resetAll();
        break;;
     case StartCode:
        startThread();
        break;
     case ScrollBarCode:
        scale = PSB.getReading();
        SBLabel.setText("   "+scale);
        break;
     default:
        break;
    } 
  }

  void writeVectorValue(double Vx, double Vy, int mode){}
  void plot(){}
}
      
    










