
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

abstract public class FourierLabGUI extends PhysicsApplet{

    FourierTransform FT, FTback;
    double[] F;
    double[] Fspectrum;
    double[] filter;
    double[] Ffinal;
    double[] Fzero;
    CurveDisplay First;
    CurveDisplay Second;
    CurveDisplay Third;
        // mouse modes
    static final int NormalMode = 0;
    static final int DrawinFirstMode = 1;
    static final int DrawinSecondMode = 2;
       // button codes
    static final int FinishDrawingCode = 1;
    static final int TransformCode = 2;
    static final int ScrollbarCode = 3;
       // scrollbar
    PhysicsScrollbar SpNorm;
    BoldLabel SpNormLabel;
    double SpScale;

    FourierLabGUI(){
	super("Fourier Lab", 256, 100, 2, 40);
        F = new double[Nx+1];
        Fspectrum = new double[Nx+1];
        filter = new double[Nx+1];
        Ffinal = new double[Nx+1];
        Fzero = new double[Nx+1];
        for (int i = 0; i <= Nx ; i++){
	    F[i] = 0.0; 
            Fspectrum[i] = 0.0; 
            filter[i] = 0.0; 
            Ffinal[i] = 0.0;
            Fzero[i] = 0.0;
        }
        FT = new FourierTransform(256);
        FTback = new FourierTransform(256);
    }
          

    void resetArrays(){
        for (int i = 0; i <= Nx ; i++){
	    F[i] = 0.0; 
            Fspectrum[i] = 0.0; 
            Ffinal[i] = 0.0;
        }
    }
        
    void resetAll(){
        for (int i = 0; i <= Nx ; i++){
	    F[i] = 0.0; 
            Fspectrum[i] = 0.0;
            filter[i] = 0.0; 
            Ffinal[i] = 0.0;
        }
    }

    void buildPicture(){
        inputWave();
        inputFilter();
        First = new CurveDisplay(100, 50, F, Fzero, 1.0, 1.0,
                           Color.blue, Color.black,DrawinFirstMode);
   	Picture.add(First,"North");
	Second = new CurveDisplay(100, 10, filter, Fspectrum, 1.0, 1.0,
			    Color.gray, Color.magenta,DrawinSecondMode);
	Picture.add(Second,"Center"); 
        Third = new CurveDisplay(100, 50, Fzero, Ffinal, 1.0, 1.0, 
			    Color.black, Color.red,203);
      	Picture.add(Third,"South");
    }

    void refreshPicture(){
        First.refresh();
        Second.refresh();
        Third.refresh();
    }

    void callTransforms(){
        for (int i = 0; i < Nx; i++){
            FT.f[i] = F[i];
        }
        FT.Transform();
        for (int i = 0; i <= Nx/2; i++){
            Fspectrum[2*i] = (5.0/Nx) * 
                       Math.sqrt( SQR(FT.fcos[i]) + SQR(FT.fsin[i]));
        }
        for (int i = 1; i < Nx; i+=2){
            Fspectrum[i] = 0.5* (Fspectrum[i-1] + Fspectrum[i+1]);
        }
        for (int i = 0 ; i <= Nx/2; i++){
            FTback.fcos[i] = FT.fcos[i] * filter[2*i];
            FTback.fsin[i] = FT.fsin[i] * filter[2*i];
        }
        FTback.InverseTransform();
        for (int i = 0; i < Nx; i++){
            Ffinal[i] = FTback.f[i];
        }
        refreshPicture();
    }

    double SQR(double a){ return a*a;}

    void buildControls(){
        Controls.setLayout(new GridLayout(0,4, 10, 10));
        ModeButton B1 = new ModeButton("Draw f", DrawinFirstMode);
        Controls.add(B1);
        ModeButton B2 = new ModeButton("Draw filter", DrawinSecondMode);
        Controls.add(B2);
        CommandButton B3 = new CommandButton("Finish drawing",
                                                           FinishDrawingCode);
        Controls.add(B3);
        CommandButton B4 = new CommandButton("Transform", TransformCode);
        Controls.add(B4);
        BoldLabel B5 = new BoldLabel("Spectrum Scale");
        Controls.add(B5);
        SpNorm = new PhysicsScrollbar(1.0, 0.2, 5.0,ScrollbarCode);
        Controls.add(SpNorm);
        SpNormLabel = new BoldLabel("    1.0");
        Controls.add(SpNormLabel);
    }

    void doAction(int Code){
        switch(Code){
	  case FinishDrawingCode:
	      refreshPicture();
              break;
	  case TransformCode:
              callTransforms();
              break;
	  case ScrollbarCode:
              SpScale = SpNorm.getReading();
              SpNormLabel.setText("   "+SpScale);
              Second.resethscale(SpScale);
              break;
	  default:
	      break;
        }
    }

    Color findColor(double A, int S){ return Color.black; }
    void writeFieldValue(double Val, int mode){}
    void writeVectorValue(double Vx, double Vy, int mode){}
    void setArrays(int i, int j, int mode){}
    void solve(){}
    void plot(){}

    abstract void inputWave();
    abstract void inputFilter();

}


