
public class myFourierTransform{

    /* an implementation of the Fast Fourier Transform in java,
         specifically for the case Nx = 256.  For such a small value
         of Nx, this program is not noticeably faster than the 
         straightforward algorithm of myFourierTransform.java   */

    int Nx = 256;
    int logN = 8;
    int N2 = 128;
    double Nd = 256.0;
    double[] f, fcos, fsin;
    double[] fcosx, fsinx, fcosy, fsiny;

    myFourierTransform(int N){
	f = new double[Nx];
        fcos = new double[N2+1];
        fsin = new double[N2+1];
        fcosx = new double[Nx];
        fsinx = new double[Nx];
        fcosy = new double[Nx];
        fsiny = new double[Nx];
    }

    void Transform(){
        int n;
        for (n = 0; n < Nx; n++){
	    fcosx[n] = f[n];
            fsinx[n] = 0.0;
        }
        FastFourierTransform();
        for (n = 0; n <= N2; n++){
            fcos[n] = fcosx[n];
            fsin[n] = fsinx[n];
        }
    }

    void InverseTransform(){
        int n;
	  /*   After complex conjugating the transform, we can simply call
	            FastFourierTransform again   */
        for (n = 0; n <= N2; n++){
	    fcosx[n] = fcos[n];
            fsinx[n] = -fsin[n];
        }
        for (n = 1; n < N2; n++){
	    fcosx[N-n] = fcos[n];
            fsinx[N-n] = fsin[n];
        }
        FastFourierTransform();
        for (n = 0; n < N; n++){
            f[n] = fcosx[n]/N;
        }
    }

    void FastFourierTransform(){
	  /*  Fourier transforms (fcosx,fsinx) and returns the result
	          in (fcosx, fsinx)   */
        int k, n, m;
        int nsize = N2;
        int msize = 2; 
        double kcos = -1.0;
        double ksin = 0.0;
        for (k = 0; k < logN; k++){
            int entry = 0;
            int mst = 0;
            int mstep = 0;
            double mcos = 1.0;
            double msin = 0.0;
            for (m = 0; m < msize;  m++){
	       for (n = 0; n < nsize ; n++){
                  fcosy[entry] = fcosx[n+mstep] 
                                     + fcosx[n+mstep + nsize] * mcos
                                     - fsinx[n+mstep + nsize] * msin;
                  fsiny[entry] = fsinx[n+mstep] 
                                     + fcosx[n+mstep + nsize] * msin
		                     + fsinx[n+mstep + nsize] * mcos;
                  entry++;
               }
               mst ++;
               if (mst >= msize/2) mst -= (msize/2);
               mstep = mst * 2 * nsize;
               double mcosx = mcos * kcos - msin * ksin;
               msin = msin * kcos + mcos * ksin;
               mcos = mcosx;
	    }
            for (n = 0; n < Nx; n++){
	       fcosx[n] = fcosy[n];
               fsinx[n] = fsiny[n];
            }
            msize *= 2;
            nsize /= 2;
            double kcosx = Math.sqrt( 0.5 * (1.0 + kcos));
            ksin = Math.sqrt( 0.5 * (1.0 - kcos));
            kcos = kcosx;
        }
    }

}

